@@ -13,12 +13,12 @@
 
 import contextlib
 import functools
-import inspect
 import operator
 import threading
 import warnings
 
-import debtcollector.removals as removals
+import debtcollector.removals
+import debtcollector.renames
 from oslo_config import cfg
 from oslo_utils import excutils
 
@@ -26,6 +26,7 @@ from oslo_db import exception
 from oslo_db import options
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import orm
+from oslo_db.sqlalchemy import utils
 
 
 class _symbol(object):
@@ -136,7 +137,7 @@ class _TransactionFactory(object):
             'sqlite_fk': _Default(False),
             'mysql_sql_mode': _Default('TRADITIONAL'),
             'mysql_enable_ndb': _Default(False),
-            'idle_timeout': _Default(3600),
+            'connection_recycle_time': _Default(3600),
             'connection_debug': _Default(0),
             'max_pool_size': _Default(),
             'max_overflow': _Default(),
@@ -177,6 +178,8 @@ class _TransactionFactory(object):
         self._legacy_facade = None
         self._start_lock = threading.Lock()
 
+    @debtcollector.renames.renamed_kwarg(
+        "idle_timeout", "connection_recycle_time", replace=True)
     def configure_defaults(self, **kw):
         """Apply default configurational options.
 
@@ -209,7 +212,7 @@ class _TransactionFactory(object):
 
         :param mysql_enable_ndb: enable MySQL Cluster (NDB) support
 
-        :param idle_timeout: connection pool recycle time,
+        :param connection_recycle_time: connection pool recycle time,
          defaults to 3600. Note the connection does not actually have to
          be "idle" to be recycled.
 
@@ -282,6 +285,8 @@ class _TransactionFactory(object):
         """
         self._configure(True, kw)
 
+    @debtcollector.renames.renamed_kwarg(
+        "idle_timeout", "connection_recycle_time", replace=True)
     def configure(self, **kw):
         """Apply configurational options.
 
@@ -529,7 +534,7 @@ class _TestTransactionFactory(_TransactionFactory):
 
     """
 
-    @removals.removed_kwarg(
+    @debtcollector.removals.removed_kwarg(
         'synchronous_reader',
         'argument value is propagated from the parent _TransactionFactory')
     def __init__(self, engine, maker, apply_global, from_factory=None, **kw):
@@ -838,7 +843,8 @@ class _TransactionContextManager(object):
         new = self._clone()
         new._root = new
         new._root_factory = self._root_factory._create_factory_copy()
-        assert not new._factory._started
+        if new._factory._started:
+            raise AssertionError('TransactionFactory is already started')
         return new
 
     def patch_factory(self, factory_or_manager):
@@ -864,7 +870,8 @@ class _TransactionContextManager(object):
             raise ValueError(
                 "_TransactionContextManager or "
                 "_TransactionFactory expected.")
-        assert self._root is self
+        if self._root is not self:
+            raise AssertionError('patch_factory only works for root factory.')
         existing_factory = self._root_factory
         self._root_factory = factory
 
@@ -965,7 +972,7 @@ class _TransactionContextManager(object):
 
     def __call__(self, fn):
         """Decorate a function."""
-        argspec = inspect.getargspec(fn)
+        argspec = utils.getargspec(fn)
         if argspec.args[0] == 'self' or argspec.args[0] == 'cls':
             context_index = 1
         else:
@@ -1196,8 +1203,8 @@ class LegacyEngineFacade(object):
     :keyword mysql_enable_ndb: If True, transparently enables support for
                                handling MySQL Cluster (NDB).
                                (defaults to False)
-    :keyword idle_timeout: timeout before idle sql connections are reaped
-                           (defaults to 3600)
+    :keyword connection_recycle_time: Time period for connections to be
+                            recycled upon checkout (defaults to 3600)
     :keyword connection_debug: verbosity of SQL debugging information.
                                -1=Off, 0=None, 100=Everything (defaults
                                to 0)
@@ -1230,7 +1237,6 @@ class LegacyEngineFacade(object):
             "oslo_db.sqlalchemy.enginefacade",
             exception.OsloDBDeprecationWarning,
             stacklevel=2)
-
         if _factory:
             self._factory = _factory
         else:
