@@ -402,8 +402,9 @@ class OpportunisticDbFixture(BaseDbFixture):
     This fixture relies upon the use of the OpportunisticDBTestMixin to supply
     a test.resources attribute, and also works much more effectively when
     combined the testresources.OptimisingTestSuite.   The
-    optimize_db_test_loader() function should be used at the module and package
-    levels to optimize database provisioning across many tests.
+    optimize_package_test_loader() function should be
+    used at the module and package levels to optimize database
+    provisioning across many tests.
 
     """
     def __init__(self, test, driver=None, ident=None):
@@ -533,7 +534,7 @@ class OpportunisticDBTestMixin(object):
             if not fixture._has_db_resource():
                 msg = fixture._get_db_resource_not_available_reason()
                 if self.SKIP_ON_UNAVAILABLE_DB:
-                    self.skip(msg)
+                    self.skipTest(msg)
                 else:
                     self.fail(msg)
 
