@@ -18,11 +18,13 @@
 
 import collections
 import contextlib
+import inspect as pyinspect
 import itertools
 import logging
 import re
 
-import debtcollector
+from alembic.migration import MigrationContext
+from alembic.operations import Operations
 from oslo_utils import timeutils
 import six
 import sqlalchemy
@@ -48,6 +50,7 @@ from sqlalchemy.types import NullType
 from oslo_db._i18n import _
 from oslo_db import exception
 from oslo_db.sqlalchemy import models
+from oslo_db.sqlalchemy import ndb
 
 # NOTE(ochuprykov): Add references for backwards compatibility
 InvalidSortKey = exception.InvalidSortKey
@@ -178,7 +181,9 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
         LOG.warning('Unique keys not in sort_keys. '
                     'The sorting order may be unstable.')
 
-    assert(not (sort_dir and sort_dirs))
+    if sort_dir and sort_dirs:
+        raise AssertionError('Disallow set sort_dir and '
+                             'sort_dirs at the same time.')
 
     # Default the sort direction to ascending
     if sort_dirs is None and sort_dir is None:
@@ -188,7 +193,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     if sort_dirs is None:
         sort_dirs = [sort_dir for _sort_key in sort_keys]
 
-    assert(len(sort_dirs) == len(sort_keys))
+    if len(sort_dirs) != len(sort_keys):
+        raise AssertionError('sort_dirs and sort_keys must have same length.')
 
     # Add sorting
     for current_sort_key, current_sort_dir in zip(sort_keys, sort_dirs):
@@ -239,7 +245,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             if marker_values[i] is not None:
                 for j in range(i):
                     model_attr = getattr(model, sort_keys[j])
-                    crit_attrs.append((model_attr == marker_values[j]))
+                    if marker_values[j] is not None:
+                        crit_attrs.append((model_attr == marker_values[j]))
 
                 model_attr = getattr(model, sort_keys[i])
                 val = marker_values[i]
@@ -248,9 +255,16 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
                     val = int(val)
                     model_attr = cast(model_attr, Integer)
                 if sort_dirs[i].startswith('desc'):
-                    crit_attrs.append((model_attr < val))
+                    crit_attr = (model_attr < val)
+                    if sort_dirs[i].endswith('nullsfirst'):
+                        crit_attr = sqlalchemy.sql.or_(crit_attr,
+                                                       model_attr.is_(None))
                 else:
-                    crit_attrs.append((model_attr > val))
+                    crit_attr = (model_attr > val)
+                    if sort_dirs[i].endswith('nullslast'):
+                        crit_attr = sqlalchemy.sql.or_(crit_attr,
+                                                       model_attr.is_(None))
+                crit_attrs.append(crit_attr)
                 criteria = sqlalchemy.sql.and_(*crit_attrs)
                 criteria_list.append(criteria)
 
@@ -433,43 +447,6 @@ def get_table(engine, name):
     return Table(name, metadata, autoload=True)
 
 
-@debtcollector.removals.removed_class(
-    'InsertFromSelect',
-    replacement='sqlalchemy.sql.expression.Insert.from_select',
-    message='this functionality is provided out-of-box by SQLAlchemy >= 1.0.0'
-)
-class InsertFromSelect(object):
-    """Form the base for `INSERT INTO table (SELECT ... )` statement.
-
-    :param table: table to insert records
-    :param select: select query
-    :param cols: list of columns to specify in insert clause
-    :return: SQLAlchemy :class:`Insert` object instance
-
-    Usage:
-
-    .. code-block:: python
-
-      select = sql.select(table_from)
-      insert = InsertFromSelect(table_to, select,
-                                ['id', 'name', 'insert_date'])
-      engine.execute(insert)
-
-    """
-    # NOTE(tdurakov): Insert from select implementation added to SQLAlchemy
-    # starting from version 0.8.7. Default SQLAlchemy implementation should be
-    # used instead of this. Deprecated.
-
-    def __new__(cls, table, select, cols=None):
-        if not cols:
-            cols = [c.name for c in table.c]
-
-        return table.insert(inline=True).from_select(cols, select)
-
-    def __init__(self, table, select, cols=None):
-        pass
-
-
 def _get_not_supported_column(col_name_col_instance, column_name):
     try:
         column = col_name_col_instance[column_name]
@@ -628,9 +605,6 @@ def _change_deleted_column_type_to_boolean_sqlite(engine, table_name,
         else:
             c_select.append(table.c.deleted == table.c.id)
 
-    ins = InsertFromSelect(new_table, sqlalchemy.sql.select(c_select))
-    engine.execute(ins)
-
     table.drop()
     for index in indexes:
         index.create(engine)
@@ -722,9 +696,6 @@ def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
         indexes.append(Index(index["name"], *column_names,
                              unique=index["unique"]))
 
-    ins = InsertFromSelect(new_table, table.select())
-    engine.execute(ins)
-
     table.drop()
     for index in indexes:
         index.create(engine)
@@ -744,54 +715,6 @@ def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
         execute()
 
 
-def get_connect_string(backend, database, user=None, passwd=None,
-                       host='localhost'):
-    """Get database connection
-
-    Try to get a connection with a very specific set of values, if we get
-    these then we'll run the tests, otherwise they are skipped
-
-    DEPRECATED: this function is deprecated and will be removed from oslo_db
-    in a few releases. Please use the provisioning system for dealing
-    with URLs and database provisioning.
-
-    """
-    args = {'backend': backend,
-            'user': user,
-            'passwd': passwd,
-            'host': host,
-            'database': database}
-    if backend == 'sqlite':
-        template = '%(backend)s:///%(database)s'
-    else:
-        template = "%(backend)s://%(user)s:%(passwd)s@%(host)s/%(database)s"
-    return template % args
-
-
-def is_backend_avail(backend, database, user=None, passwd=None):
-    """Return True if the given backend is available.
-
-
-    DEPRECATED: this function is deprecated and will be removed from oslo_db
-    in a few releases. Please use the provisioning system to access
-    databases based on backend availability.
-
-    """
-    from oslo_db.sqlalchemy import provision
-
-    connect_uri = get_connect_string(backend=backend,
-                                     database=database,
-                                     user=user,
-                                     passwd=passwd)
-    try:
-        eng = provision.Backend._ensure_backend_available(connect_uri)
-        eng.dispose()
-    except exception.BackendNotAvailable:
-        return False
-    else:
-        return True
-
-
 def get_db_connection_info(conn_pieces):
     database = conn_pieces.path.strip('/')
     loc_pieces = conn_pieces.netloc.split('@')
@@ -1111,11 +1034,10 @@ def get_non_innodb_tables(connectable, skip_tables=('migrate_version',
                                                     'alembic_version')):
     """Get a list of tables which don't use InnoDB storage engine.
 
-     :param connectable: a SQLAlchemy Engine or a Connection instance
-     :param skip_tables: a list of tables which might have a different
-                         storage engine
-     """
-
+    :param connectable: a SQLAlchemy Engine or a Connection instance
+    :param skip_tables: a list of tables which might have a different
+                        storage engine
+    """
     query_str = """
         SELECT table_name
         FROM information_schema.tables
@@ -1169,6 +1091,118 @@ def get_non_ndbcluster_tables(connectable, skip_tables=None):
     return [i[0] for i in nonndbcluster]
 
 
+def get_foreign_key_constraint_name(engine, table_name, column_name):
+    """Find the name of foreign key in a table, given constrained column name.
+
+    :param engine: a SQLAlchemy engine (or connection)
+
+    :param table_name: name of table which contains the constraint
+
+    :param column_name: name of column that is constrained by the foreign key.
+
+    :return: the name of the first foreign key constraint which constrains
+     the given column in the given table.
+
+    """
+    insp = inspect(engine)
+    for fk in insp.get_foreign_keys(table_name):
+        if column_name in fk['constrained_columns']:
+            return fk['name']
+
+
+@contextlib.contextmanager
+def suspend_fk_constraints_for_col_alter(
+        engine, table_name, column_name, referents=[]):
+    """Detect foreign key constraints, drop, and recreate.
+
+    This is used to guard against a column ALTER that on some backends
+    cannot proceed unless foreign key constraints are not present.
+
+    e.g.::
+
+        from oslo_db.sqlalchemy.util import (
+            suspend_fk_constraints_for_col_alter
+        )
+
+        with suspend_fk_constraints_for_col_alter(
+            migrate_engine, "user_table",
+            referents=[
+                "local_user", "nonlocal_user", "project"
+            ]):
+            user_table.c.domain_id.alter(nullable=False)
+
+    :param engine: a SQLAlchemy engine (or connection)
+
+    :param table_name: target table name.  All foreign key constraints
+     that refer to the table_name / column_name will be dropped and recreated.
+
+    :param column_name: target column name.  all foreign key constraints
+     which refer to this column, either partially or fully, will be dropped
+     and recreated.
+
+    :param referents: sequence of string table names to search for foreign
+     key constraints.   A future version of this function may no longer
+     require this argument, however for the moment it is required.
+
+    """
+    if (
+        not ndb.ndb_status(engine)
+    ):
+        yield
+    else:
+        with engine.connect() as conn:
+            insp = inspect(conn)
+            fks = []
+            for ref_table_name in referents:
+                for fk in insp.get_foreign_keys(ref_table_name):
+                    if not fk.get('name'):
+                        raise AssertionError("foreign key hasn't a name.")
+                    if fk['referred_table'] == table_name and \
+                            column_name in fk['referred_columns']:
+                        fk['source_table'] = ref_table_name
+                        if 'options' not in fk:
+                            fk['options'] = {}
+                        fks.append(fk)
+
+            ctx = MigrationContext.configure(conn)
+            op = Operations(ctx)
+
+            for fk in fks:
+                op.drop_constraint(
+                    fk['name'], fk['source_table'], type_="foreignkey")
+            yield
+            for fk in fks:
+                op.create_foreign_key(
+                    fk['name'], fk['source_table'],
+                    fk['referred_table'],
+                    fk['constrained_columns'],
+                    fk['referred_columns'],
+                    onupdate=fk['options'].get('onupdate'),
+                    ondelete=fk['options'].get('ondelete'),
+                    deferrable=fk['options'].get('deferrable'),
+                    initially=fk['options'].get('initially'),
+                )
+
+
+def getargspec(fn):
+    """Inspects a function for its argspec.
+
+    This is to handle a difference between py2/3. The Python 2.x getargspec
+    call is deprecated in Python 3.x, with the suggestion to use the signature
+    call instead.
+
+    To keep compatibility with the results, while avoiding deprecation
+    warnings, this instead will use the getfullargspec instead.
+
+    :param fn: The function to inspect.
+    :returns: The argspec for the function.
+    """
+    if hasattr(pyinspect, 'getfullargspec'):
+        return pyinspect.getfullargspec(fn)
+
+    return pyinspect.getargspec(fn)
+
+
 class NonCommittingConnectable(object):
     """A ``Connectable`` substitute which rolls all operations back.
 
