@@ -926,7 +926,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 getattr, context, 'session'
             )
 
-            self.assertRegexpMatches(
+            self.assertRegex(
                 exc.args[0],
                 "The 'session' context attribute was requested but it has "
                 "not been established for this context."
@@ -941,7 +941,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
                 getattr, context, 'connection'
             )
 
-            self.assertRegexpMatches(
+            self.assertRegex(
                 exc.args[0],
                 "The 'connection' context attribute was requested but it has "
                 "not been established for this context."
@@ -954,7 +954,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             getattr, context, 'session'
         )
 
-        self.assertRegexpMatches(
+        self.assertRegex(
             exc.args[0],
             "No TransactionContext is established for "
             "this .*RequestContext.* object within the current "
@@ -972,7 +972,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             getattr, context, 'connection'
         )
 
-        self.assertRegexpMatches(
+        self.assertRegex(
             exc.args[0],
             "No TransactionContext is established for "
             "this .*RequestContext.* object within the current "
@@ -990,7 +990,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             getattr, context, 'transaction'
         )
 
-        self.assertRegexpMatches(
+        self.assertRegex(
             exc.args[0],
             "No TransactionContext is established for "
             "this .*RequestContext.* object within the current "
@@ -1008,7 +1008,7 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             getattr, context, 'transaction_ctx'
         )
 
-        self.assertRegexpMatches(
+        self.assertRegex(
             exc.args[0],
             "No TransactionContext is established for "
             "this .*RequestContext.* object within the current "
@@ -2156,7 +2156,8 @@ class ConfigOptionsTest(oslo_test_base.BaseTestCase):
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always")
 
-            factory.configure(fake1='x', idle_timeout=200, wrong2='y')
+            factory.configure(
+                fake1='x', connection_recycle_time=200, wrong2='y')
 
         self.assertEqual(1, len(w))
         self.assertTrue(
